/*
 * TCS Dependency Map Renderer
 *
 * Copyright (c) 2005 TopCoder, Inc. All rights reserved.
 *
 * Author: Pops, TCSDeveloper
 */

package com.topcoder.rules.dependencymap.view.stresstests;

import junit.framework.TestCase;
import com.topcoder.rules.dependencymap.view.*;
import java.util.*;
import java.io.File;


/**
*   Stress tests Task 6.
*/
public class StressRenderAll extends TestCase
{
    /**
    * Dependencies purposely built during setup to have
    * a large number of sub-elements
    */
    Dependency myDependency, linearDependency;

    /**
    * number of levels the test dependency tree will have
    */
    private final int maxNumLevels = 4;

    /**
     * number of levels used to test the renderAll method
     */
    private final int maxLinearLevels = 1000;

    /**
    * number of sub-elements for each level of the test dependency
     * this number is even so as to avoid the blank column issue
    */
    private final int maxNumElements = 11;

    /**
    * Test Diagram renderer
    */
    private DiagramRenderer sampleRenderer;

    /**
    * Template name for the test files
    */
    private final String fileName = "../../test_files/file00";

    /**
    * counter for number of dependants
    */
    private int totalCount;

    public void setUp()
    {
        sampleRenderer = new SVGRenderer(new File(fileName));
        //create the sample dependency graph
        Dependency configManager = new Dependency("Config Manager");
        Dependency spellCheck = new Dependency("Spell Check");
        spellCheck.addDependant(configManager);

        //create the large dependency graph
        totalCount = 1;
        myDependency = new Dependency("largeDependency");
        Vector currentLevel = new Stack();
        currentLevel.add(myDependency);
        for(int i = 0; i < maxNumLevels; ++i)
        {
            Vector nextLevel = new Stack();
            while(!currentLevel.isEmpty())
            {
                Dependency current = (Dependency)currentLevel.firstElement();
                currentLevel.removeElementAt(0);
                for(int j = 0; j < maxNumElements; ++j)
                {
                    Dependency d = new Dependency("level " + i + " element " + j);
                    current.addDependant(d);
                    nextLevel.add(d);
                    ++totalCount;
                }
            }
            currentLevel = new Vector(nextLevel);
        }

        linearDependency = new Dependency("linear dependency");
        Dependency parent = linearDependency;
        for(int i = 0; i < maxLinearLevels; ++i)
        {
            Dependency current = new Dependency("level" + i);
            parent.addDependant(current);
            parent = current;
        }
    }

    /**
    * Attempts to render a large dependency graph
    */
    public void testLargeGraph() throws RendererException
    {
        // Create the Dependency Renderer and add a diagram renderer
        DependencyRenderer renderer = new DependencyRenderer();
        renderer.addDiagramRenderer(sampleRenderer);
        long startTime = new Date().getTime();
        // Render the dependency
        renderer.renderAll(myDependency, true);
        long ts = new Date().getTime() - startTime;
        System.out.println("Rendering all nodes of a graph with " + maxNumLevels + " levels and " + maxNumElements + " elements per level  (total of " + totalCount + " elements) took " + ts + " ms");
    }

    public void testLongLinearGraph() throws RendererException
    {
        // Create the Dependency Renderer and add a diagram renderer
        DependencyRenderer renderer = new DependencyRenderer();
        renderer.addDiagramRenderer(sampleRenderer);
        long startTime = new Date().getTime();
        // Render the dependency
        renderer.renderAll(linearDependency, true);
        long ts = new Date().getTime() - startTime;
        System.out.println("Rendering a dependency graph with " + maxLinearLevels + " levels took " + ts + " ms");
    }
}
